@@ -239,19 +239,19 @@ plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 	table = gtk_table_new (3, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (ui->vbox), table, TRUE, TRUE, 0);
 
-	ui->user_db_button = gtk_radio_button_new_with_label (NULL, _("User local database"));
+	ui->user_db_button = gtk_radio_button_new_with_label (NULL, _("Desktop CouchDB"));
 	gtk_table_attach (GTK_TABLE (table), ui->user_db_button, 0, 2, 0, 1,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
 	ui->system_db_button = gtk_radio_button_new_with_label (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (ui->user_db_button)),
-		_("System wide database"));
+		_("System-wide CouchDB"));
 	gtk_table_attach (GTK_TABLE (table), ui->system_db_button, 0, 2, 1, 2,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
 	ui->remote_db_button = gtk_radio_button_new_with_label (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (ui->user_db_button)),
-		_("Remote database"));
+		_("Remote CouchDB server"));
 	gtk_table_attach (GTK_TABLE (table), ui->remote_db_button, 0, 1, 2, 3,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 	ui->remote_db_entry = gtk_entry_new ();
